/***************************************************************************
 *            1wire.c
 *
 *  Mon Apr 27 23:28:56 2009
 *  Copyright  2009  Dirk Broßwick
 *  <sharandac@snafu.de>
 *
 * Der Code ist in Anlehnung an Peter Dannegger seinem Code geschrieben.
 * Veröffentlicht unter http://www.mikrocontroller.net/topic/14792 .
 ****************************************************************************/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 

#include <avr/io.h>
#include <avr/interrupt.h>

#include "hardware/timer1/timer1.h"

#include "1wire.h"

char ONEWIRE_reset( void )
{
	char sreg_tmp;
	char Error;
	
	DQ_OUT &= ~(1<<DQ_PIN);
	DQ_DDR |= 1<<DQ_PIN;
	
	sreg_tmp = SREG;
	cli();
	
	timer1_wait( 960 );

	DQ_DDR &= ~(1<<DQ_PIN);
	
	timer1_wait( 132 );
	
	Error = DQ_IN & ( 1<<DQ_PIN );

	SREG = sreg_tmp;

	timer1_wait( 828 );
	
	if( ( DQ_IN & ( 1<<DQ_PIN ) ) == 0 )
		Error = 1;
	
	return Error;
}

char ONEWIRE_bitio( char BIT )
{
	char sreg_tmp;
	sreg_tmp = SREG;
	cli();
	
	DQ_DDR |= 1<<DQ_PIN;
	timer1_wait( 2 );

	if( BIT )
		DQ_DDR &= ~(1<<DQ_PIN);
	
	timer1_wait( 28 );
	
	if( (DQ_IN & ( 1<<DQ_PIN ) ) == 0 )
		BIT = 0;
	
	timer1_wait( 90 );
	DQ_DDR &= ~( 1<<DQ_PIN );

	SREG = sreg_tmp;

	return BIT;
}

char ONEWIRE_writebyte( char BYTE )
{
	char i = 8, j;

	do{
		j = ONEWIRE_bitio( BYTE & 1 );
		BYTE >>= 1;
		if( j )
		BYTE |= 0x80;
	}while( --i );
  
	return BYTE;
}

char ONEWIRE_readbyte( void )
{
  return ONEWIRE_writebyte( 0xFF );
}

char ONEWIRE_searchrom( char diff, char * id )
{	
	char j, next_diff, b;

	int i;
	if( ONEWIRE_reset() )	
		return PRESENCE_ERR;

	ONEWIRE_writebyte( SEARCH_ROM );
	next_diff = LAST_DEVICE;

	i = 64;
	do{
		j = 8;
		do{
		b = ONEWIRE_bitio( 1 );
		if( ONEWIRE_bitio( 1 ) )
		{
			if( b )
				return DATA_ERR;
  		}
		else
		{
			if( !b )
			{
				if( diff > i || ((*id & 1) && diff != i) )
				{
					b = 1;
					next_diff = i;
				}
			}
		}
 
				
		ONEWIRE_bitio( b );
		*id >>= 1;
		if( b )
			*id |= 0x80;
  		i--;
		}while( --j );
		

		id++;
	
	}while( i );
	
	return next_diff;
}

void ONEWIRE_command( char command, char * id )
{
	char i;
	
	ONEWIRE_reset();
	if( id )
	{
		ONEWIRE_writebyte( MATCH_ROM );
		i = 8;
		do{
			ONEWIRE_writebyte( * id );
  			id++;
		}while( --i );
	}
	else
	{
		ONEWIRE_writebyte( SKIP_ROM );
	}
	ONEWIRE_writebyte( command );
}
